#include "ADXL345_accel.h"
#include "imu_int.h"
#include "ram.h"

// accelerometer registers
#define ACCEL_THRESH_TAP                 0x1D
#define ACCEL_OFSX                       0x1E
#define ACCEL_OFSY                       0x1F
#define ACCEL_OFSZ                       0x20
#define ACCEL_DUR                        0x21
#define ACCEL_LATENT                     0x22
#define ACCEL_WINDOW                     0x23
#define ACCEL_THRESH_ACT                 0x24
#define ACCEL_THRESH_INACT               0x25
#define ACCEL_TIME_INACT                 0x26
#define ACCEL_ACT_INACT_CTL              0x27
#define ACCEL_THRESH_FF                  0x28
#define ACCEL_TIME_FF                    0x29
#define ACCEL_TAP_AXES                   0x2A
#define ACCEL_ACT_TAP_STATUS             0x2B
#define ACCEL_BW_RATE                    0x2C
#define ACCEL_POWER_CTL                  0x2D
#define ACCEL_INT_ENABLE                 0x2E
#define ACCEL_INT_MAP                    0x2F
#define ACCEL_INT_SOURCE                 0x30
#define ACCEL_DATA_FORMAT                0x31

// accelerometer configuration data
#define ACCEL_RATE_3200HZ            0x0F
#define ACCEL_RATE_1600HZ            0x0E
#define ACCEL_RATE_800HZ             0x0D
#define ACCEL_RATE_400HZ             0x0C
#define ACCEL_RATE_200HZ             0x0B
#define ACCEL_RATE_100HZ             0x0A
#define ACCEL_RATE_50HZ              0x09
#define ACCEL_RATE_25HZ              0x08
#define ACCEL_RATE_12_5HZ            0x07
#define ACCEL_RATE_6_25HZ            0x06

#define ACCEL_ENABLE_LINK            0x20
#define ACCEL_ENABLE_AUTOSLEEP       0x10
#define ACCEL_ENABLE_MEASURE         0x08
#define ACCEL_ENABLE_SLEEP           0x04
#define ACCEL_SLEEP_8HZ              0x00
#define ACCEL_SLEEP_4HZ              0x01
#define ACCEL_SLEEP_2HZ              0x02
#define ACCEL_SLEEP_1HZ              0x03

#define ACCEL_ENABLE_FULL_RES        0x08
#define ACCEL_JUSTIFY_LEFT           0x04
#define ACCEL_JUSTIFY_RIGHT          0x00
#define ACCEL_RANGE_2G               0x00
#define ACCEL_RANGE_4G               0x01
#define ACCEL_RANGE_8G               0x02
#define ACCEL_RANGE_16G              0x03

#define ACCEL_FIFO_BYPASS            0x00
#define ACCEL_FIFO_BUFFER            0x40
#define ACCEL_FIFO_STREAM            0x80
#define ACCEL_FIFO_TRIGGER           0xC0

// private variables
// accelerometer data and configuration
const uint8_t accel_id_reg=0x00;
const uint8_t accel_id=0xE5;
const uint8_t accel_id_len=0x01;
const uint8_t accel_conf_reg=0x1D;
const uint8_t accel_conf_len=0x15;
const uint8_t accel_data_reg=0x32;
const uint8_t accel_data_len=0x06;
const uint8_t accel_fifo_reg=0x38;
const uint8_t accel_fifo_len=0x01;
const uint8_t accel_status_reg=0x39;
const uint8_t accel_status_len=0x01;
const uint8_t accel_rd_addr=0xA6;
const uint8_t accel_wr_addr=0xA7;
uint8_t i2c_accel_conf[21];
uint8_t i2c_accel_fifo_conf;
uint8_t i2c_accel_data[6];
uint8_t i2c_accel_status;

// public accelerometer functions
void imu_accel_get_config(void)
{
  i2c_op=I2C_RD_OP_CMD;
  IMU_DMA_RX_InitStructure.DMA_BufferSize=accel_conf_len;
  IMU_DMA_RX_InitStructure.DMA_Memory0BaseAddr=(uint32_t)i2c_accel_conf;
  DMA_Init(I2C_IMU_DMA_STREAM_RX,&IMU_DMA_RX_InitStructure);
  IMU_DMA_TX_InitStructure.DMA_BufferSize=1;
  IMU_DMA_TX_InitStructure.DMA_Memory0BaseAddr=(uint32_t)&accel_conf_reg;

  i2c_rd_addr=accel_rd_addr;
  i2c_wr_addr=accel_wr_addr;
  I2C_GenerateSTART(I2C_IMU, ENABLE);
  imu_wait_op_done();
}

void imu_accel_start(void)
{
  // enter measure mode
  i2c_accel_conf[ACCEL_POWER_CTL-ACCEL_THRESH_TAP]|=ACCEL_ENABLE_MEASURE;
  i2c_data_out[0]=ACCEL_POWER_CTL;
  i2c_data_out[1]=i2c_accel_conf[ACCEL_POWER_CTL-ACCEL_THRESH_TAP];
  // write the fifo configuration
  i2c_op=I2C_WR_OP;
  IMU_DMA_TX_InitStructure.DMA_BufferSize=2;
  IMU_DMA_TX_InitStructure.DMA_Memory0BaseAddr = (uint32_t)i2c_data_out;
  DMA_Init(I2C_IMU_DMA_STREAM_TX,&IMU_DMA_TX_InitStructure);
  i2c_rd_addr=accel_rd_addr;
  i2c_wr_addr=accel_wr_addr;
  I2C_GenerateSTART(I2C_IMU, ENABLE);
  imu_wait_op_done();
}

void imu_accel_stop(void)
{
  // enter measure mode
  i2c_accel_conf[ACCEL_POWER_CTL-ACCEL_THRESH_TAP]&=~ACCEL_ENABLE_MEASURE;
  i2c_data_out[0]=ACCEL_POWER_CTL;
  i2c_data_out[1]=i2c_accel_conf[ACCEL_POWER_CTL-ACCEL_THRESH_TAP];
  // write the fifo configuration
  i2c_op=I2C_WR_OP;
  IMU_DMA_TX_InitStructure.DMA_BufferSize=2;
  IMU_DMA_TX_InitStructure.DMA_Memory0BaseAddr = (uint32_t)i2c_data_out;
  DMA_Init(I2C_IMU_DMA_STREAM_TX,&IMU_DMA_TX_InitStructure);
  i2c_rd_addr=accel_rd_addr;
  i2c_wr_addr=accel_wr_addr;
  I2C_GenerateSTART(I2C_IMU, ENABLE);
  imu_wait_op_done();
}

void imu_accel_get_data(void)
{
  i2c_op=I2C_RD_OP_CMD;
  IMU_DMA_RX_InitStructure.DMA_BufferSize=accel_data_len;
  IMU_DMA_RX_InitStructure.DMA_Memory0BaseAddr=(uint32_t)i2c_accel_data;
  DMA_Init(I2C_IMU_DMA_STREAM_RX,&IMU_DMA_RX_InitStructure);
  IMU_DMA_TX_InitStructure.DMA_BufferSize=1;
  IMU_DMA_TX_InitStructure.DMA_Memory0BaseAddr=(uint32_t)&accel_data_reg;
  DMA_Init(I2C_IMU_DMA_STREAM_TX,&IMU_DMA_TX_InitStructure);
  i2c_rd_addr=accel_rd_addr;
  i2c_wr_addr=accel_wr_addr;
  I2C_GenerateSTART(I2C_IMU, ENABLE);
}

void imu_accel_set_config(void)
{
  uint8_t i;

  // set the configuration
  i2c_accel_conf[ACCEL_BW_RATE-ACCEL_THRESH_TAP]|=ACCEL_RATE_100HZ;
  i2c_accel_conf[ACCEL_DATA_FORMAT-ACCEL_THRESH_TAP]&=0xF0;
  i2c_accel_conf[ACCEL_DATA_FORMAT-ACCEL_THRESH_TAP]|=ACCEL_RANGE_8G;// +/-8g range
  i2c_accel_conf[ACCEL_DATA_FORMAT-ACCEL_THRESH_TAP]|=ACCEL_JUSTIFY_RIGHT;// justify right
  i2c_accel_conf[ACCEL_DATA_FORMAT-ACCEL_THRESH_TAP]|=ACCEL_ENABLE_FULL_RES;// enable full resolution according to the range
  // write the configuration
  for(i=0;i<accel_conf_len;i++)
    i2c_data_out[i+1]=i2c_accel_conf[i];
  i2c_data_out[0]=accel_conf_reg;
  i2c_op=I2C_WR_OP;
  IMU_DMA_TX_InitStructure.DMA_BufferSize=accel_conf_len+1;
  IMU_DMA_TX_InitStructure.DMA_Memory0BaseAddr = (uint32_t)i2c_data_out;
  DMA_Init(I2C_IMU_DMA_STREAM_TX,&IMU_DMA_TX_InitStructure);
  i2c_rd_addr=accel_rd_addr;
  i2c_wr_addr=accel_wr_addr;
  I2C_GenerateSTART(I2C_IMU, ENABLE);
  imu_wait_op_done();

  // set the fifo configuration
  i2c_accel_fifo_conf=ACCEL_FIFO_STREAM;
  i2c_data_out[0]=accel_fifo_reg;
  i2c_data_out[1]=i2c_accel_fifo_conf;
  // write the fifo configuration
  i2c_op=I2C_WR_OP;
  IMU_DMA_TX_InitStructure.DMA_BufferSize=accel_fifo_len+1;
  IMU_DMA_TX_InitStructure.DMA_Memory0BaseAddr = (uint32_t)i2c_data_out;
  DMA_Init(I2C_IMU_DMA_STREAM_TX,&IMU_DMA_TX_InitStructure);
  i2c_rd_addr=accel_rd_addr;
  i2c_wr_addr=accel_wr_addr;
  I2C_GenerateSTART(I2C_IMU, ENABLE);
  imu_wait_op_done();
}

uint8_t imu_accel_detect(void)
{
  i2c_accel_data[0]=0x00;
  i2c_op=I2C_RD_OP_CMD;
  IMU_DMA_RX_InitStructure.DMA_BufferSize=accel_id_len;
  IMU_DMA_RX_InitStructure.DMA_Memory0BaseAddr=(uint32_t)i2c_accel_data;
  DMA_Init(I2C_IMU_DMA_STREAM_RX,&IMU_DMA_RX_InitStructure);
  IMU_DMA_TX_InitStructure.DMA_BufferSize=1;
  IMU_DMA_TX_InitStructure.DMA_Memory0BaseAddr=(uint32_t)&accel_id_reg;
  DMA_Init(I2C_IMU_DMA_STREAM_TX,&IMU_DMA_TX_InitStructure);
  i2c_rd_addr=accel_rd_addr;
  i2c_wr_addr=accel_wr_addr;
  I2C_GenerateSTART(I2C_IMU, ENABLE);
  imu_wait_op_done();
  if(i2c_accel_data[0]==accel_id)
  {
    imu_accel_get_config();
    imu_accel_set_config();
    return 0x01;
  }
  else
    return 0x00;
}

void imu_accel_process_data(void)
{
  uint8_t i;
  int16_t value;

  for(i=0;i<3;i++)
  {
    value=((int16_t)(i2c_accel_data[i*2]+(i2c_accel_data[i*2+1]<<8)))<<2;
    ram_data[BIOLOID_IMU_ACCEL_X_L+i*2]=value&0xFF;
    ram_data[BIOLOID_IMU_ACCEL_X_H+i*2]=(value>>8);
  }
}

