#include "ITG3200_gyro.h"
#include "imu_int.h"
#include "ram.h"

// gyroscope registers
#define GYRO_SMPL_DIV                    0x15
#define GYRO_CONFIG                      0x16
#define GYRO_INT_CONFIG                  0x17
#define GYRO_STATUS                      0x1A
#define GYRO_POWER_MGM                   0x3E

// configuration data
#define GYRO_FS_2000                     0x18
#define GYRO_BW_256                      0x00
#define GYRO_BW_188                      0x01
#define GYRO_BW_98                       0x02
#define GYRO_BW_42                       0x03
#define GYRO_BW_20                       0x04
#define GYRO_BW_10                       0x05
#define GYRO_BW_5                        0x06

#define GYRO_INT_ACTIVE_LOW              0x80
#define GYRO_INT_ACTIVE_HIGH             0x00
#define GYRO_INT_OPEN_DRAIN              0x40                  
#define GYRO_INT_PUSH_PULL               0x00                  
#define GYRO_INT_LATCH                   0x20
#define GYRO_INT_PULSE                   0x00
#define GYRO_INT_CLEAR_RD_ANY            0x10
#define GYRO_INT_CLEAR_RD_STATUS         0x00
#define GYRO_INT_DEVICE_READY            0x04
#define GYRO_INT_DATA_READY              0x01

#define GYRO_INT_DEVICE_READY_FLAG       0x04
#define GYRO_INT_DATA_READY_FLAG         0x01

#define GYRO_RESET                       0x80
#define GYRO_SLEEP_MODE                  0x40
#define GYRO_STBY_X                      0x20
#define GYRO_STBY_Y                      0x10
#define GYRO_STBY_Z                      0x08
#define GYRO_CLK_INT                     0x00
#define GYRO_CLK_PLL_X                   0x01
#define GYRO_CLK_PLL_Y                   0x02
#define GYRO_CLK_PLL_Z                   0x03
#define GYRO_CLK_PLL_32768               0x04
#define GYRO_CLK_PLL_19_2M               0x05

// gyroscope data and configuration
const uint8_t gyro_id_reg=0x00;
const uint8_t gyro_id=0x68;
const uint8_t gyro_id_len=0x01;
const uint8_t gyro_conf_reg=0x15;
const uint8_t gyro_conf_len=0x03;
const uint8_t gyro_status_reg=0x1A;
const uint8_t gyro_status_len=0x01;
const uint8_t gyro_data_reg=0x1B;
const uint8_t gyro_data_len=0x08;
const uint8_t gyro_power_reg=0x3E;
const uint8_t gyro_power_len=0x01;
const uint8_t gyro_rd_addr=0xD0;
const uint8_t gyro_wr_addr=0xD1;
uint8_t i2c_gyro_conf[3];
uint8_t i2c_gyro_data[8];
uint8_t i2c_gyro_power_mgm;
uint8_t i2c_gyro_status;

// gyroscope private functions
uint8_t imu_gyro_detect(void)
{
  i2c_op=I2C_RD_OP_CMD;
  IMU_DMA_RX_InitStructure.DMA_BufferSize=gyro_id_len;
  IMU_DMA_RX_InitStructure.DMA_Memory0BaseAddr = (uint32_t)i2c_gyro_data;
  DMA_Init(I2C_IMU_DMA_STREAM_RX,&IMU_DMA_RX_InitStructure);
  IMU_DMA_TX_InitStructure.DMA_BufferSize=1;
  IMU_DMA_TX_InitStructure.DMA_Memory0BaseAddr = (uint32_t)&gyro_id_reg;
  DMA_Init(I2C_IMU_DMA_STREAM_TX,&IMU_DMA_TX_InitStructure);
  i2c_gyro_data[0]=0x00;
  i2c_rd_addr=gyro_rd_addr;
  i2c_wr_addr=gyro_wr_addr;
  I2C_GenerateSTART(I2C_IMU, ENABLE);

  imu_wait_op_done();
  if((i2c_gyro_data[0]&0x7E)==gyro_id)
  {
    imu_gyro_get_config();
    imu_gyro_set_config();
    return 0x01;
  }
  else
    return 0x00;
}

void imu_gyro_get_config(void)
{
  /* read the configuration */
  i2c_op=I2C_RD_OP_CMD;
  IMU_DMA_RX_InitStructure.DMA_BufferSize=gyro_conf_len;
  IMU_DMA_RX_InitStructure.DMA_Memory0BaseAddr=(uint32_t)i2c_gyro_conf;
  DMA_Init(I2C_IMU_DMA_STREAM_RX,&IMU_DMA_RX_InitStructure);
  IMU_DMA_TX_InitStructure.DMA_BufferSize=1;
  IMU_DMA_TX_InitStructure.DMA_Memory0BaseAddr=(uint32_t)&gyro_conf_reg;
  i2c_rd_addr=gyro_rd_addr;
  i2c_wr_addr=gyro_wr_addr;
  I2C_GenerateSTART(I2C_IMU, ENABLE);
  imu_wait_op_done();

  /* read trhe power management */
  i2c_op=I2C_RD_OP_CMD;
  IMU_DMA_RX_InitStructure.DMA_BufferSize=gyro_power_len;
  IMU_DMA_RX_InitStructure.DMA_Memory0BaseAddr=(uint32_t)i2c_gyro_power_mgm;
  DMA_Init(I2C_IMU_DMA_STREAM_RX,&IMU_DMA_RX_InitStructure);
  IMU_DMA_TX_InitStructure.DMA_BufferSize=1;
  IMU_DMA_TX_InitStructure.DMA_Memory0BaseAddr=(uint32_t)&gyro_power_reg;
  i2c_rd_addr=gyro_rd_addr;
  i2c_wr_addr=gyro_wr_addr;
  I2C_GenerateSTART(I2C_IMU, ENABLE);
  imu_wait_op_done();
}

void imu_gyro_start(void)
{
  // enter measure mode
  i2c_gyro_power_mgm&=~(GYRO_SLEEP_MODE | GYRO_STBY_X | GYRO_STBY_Y | GYRO_STBY_Z);
  i2c_data_out[0]=GYRO_POWER_MGM;
  i2c_data_out[1]=i2c_gyro_power_mgm;
  // write the fifo configuration
  i2c_op=I2C_WR_OP;
  IMU_DMA_TX_InitStructure.DMA_BufferSize=2;
  IMU_DMA_TX_InitStructure.DMA_Memory0BaseAddr = (uint32_t)i2c_data_out;
  DMA_Init(I2C_IMU_DMA_STREAM_TX,&IMU_DMA_TX_InitStructure);
  i2c_rd_addr=gyro_rd_addr;
  i2c_wr_addr=gyro_wr_addr;
  I2C_GenerateSTART(I2C_IMU, ENABLE);
  imu_wait_op_done();
}

void imu_gyro_stop(void)
{
  // enter measure mode
  i2c_gyro_power_mgm|=(GYRO_SLEEP_MODE | GYRO_STBY_X | GYRO_STBY_Y | GYRO_STBY_Z);
  i2c_data_out[0]=GYRO_POWER_MGM;
  i2c_data_out[1]=i2c_gyro_power_mgm;
  // write the fifo configuration
  i2c_op=I2C_WR_OP;
  IMU_DMA_TX_InitStructure.DMA_BufferSize=2;
  IMU_DMA_TX_InitStructure.DMA_Memory0BaseAddr = (uint32_t)i2c_data_out;
  DMA_Init(I2C_IMU_DMA_STREAM_TX,&IMU_DMA_TX_InitStructure);
  i2c_rd_addr=gyro_rd_addr;
  i2c_wr_addr=gyro_wr_addr;
  I2C_GenerateSTART(I2C_IMU, ENABLE);
  imu_wait_op_done();
}

void imu_gyro_get_data(void)
{
  i2c_op=I2C_RD_OP_CMD;
  IMU_DMA_RX_InitStructure.DMA_BufferSize=gyro_data_len;
  IMU_DMA_RX_InitStructure.DMA_Memory0BaseAddr=(uint32_t)i2c_gyro_data;
  DMA_Init(I2C_IMU_DMA_STREAM_RX,&IMU_DMA_RX_InitStructure);
  IMU_DMA_TX_InitStructure.DMA_BufferSize=1;
  IMU_DMA_TX_InitStructure.DMA_Memory0BaseAddr=(uint32_t)&gyro_data_reg;
  DMA_Init(I2C_IMU_DMA_STREAM_TX,&IMU_DMA_TX_InitStructure);
  i2c_rd_addr=gyro_rd_addr;
  i2c_wr_addr=gyro_wr_addr;
  I2C_GenerateSTART(I2C_IMU, ENABLE);
}

void imu_gyro_set_config(void)
{
  uint8_t i;

  // set the configuration
  i2c_gyro_conf[GYRO_SMPL_DIV-GYRO_SMPL_DIV]=9;// 100 Hz
  i2c_gyro_conf[GYRO_CONFIG-GYRO_SMPL_DIV]=GYRO_FS_2000 | GYRO_BW_188;
  i2c_gyro_conf[GYRO_INT_CONFIG-GYRO_SMPL_DIV]=0x00;// no interrupts enabled
  // write the configuration
  for(i=0;i<gyro_conf_len;i++)
    i2c_data_out[i+1]=i2c_gyro_conf[i];
  i2c_data_out[0]=gyro_conf_reg;
  i2c_op=I2C_WR_OP;
  IMU_DMA_TX_InitStructure.DMA_BufferSize=gyro_conf_len+1;
  IMU_DMA_TX_InitStructure.DMA_Memory0BaseAddr = (uint32_t)i2c_data_out;
  DMA_Init(I2C_IMU_DMA_STREAM_TX,&IMU_DMA_TX_InitStructure);
  i2c_rd_addr=gyro_rd_addr;
  i2c_wr_addr=gyro_wr_addr;
  I2C_GenerateSTART(I2C_IMU, ENABLE);
  imu_wait_op_done();

  // set the fifo configuration
  i2c_gyro_power_mgm|=GYRO_CLK_INT;
  i2c_data_out[0]=gyro_power_reg;
  i2c_data_out[1]=i2c_gyro_power_mgm;
  // write the fifo configuration
  i2c_op=I2C_WR_OP;
  IMU_DMA_TX_InitStructure.DMA_BufferSize=gyro_power_len+1;
  IMU_DMA_TX_InitStructure.DMA_Memory0BaseAddr = (uint32_t)i2c_data_out;
  DMA_Init(I2C_IMU_DMA_STREAM_TX,&IMU_DMA_TX_InitStructure);
  i2c_rd_addr=gyro_rd_addr;
  i2c_wr_addr=gyro_wr_addr;
  I2C_GenerateSTART(I2C_IMU, ENABLE);
  imu_wait_op_done();
}

void imu_gyro_process_data(void)
{
  uint8_t i;
  int16_t value;

  // process temperature
  ram_data[BIOLOID_IMU_TEMP]=((((int16_t)(i2c_gyro_data[1]+(i2c_gyro_data[0]<<8)))-13200)/280)+35;
  // process gyro data
  for(i=0;i<3;i++)
  {
    value=(((int16_t)(i2c_gyro_data[i*2+3]+(i2c_gyro_data[i*2+2]<<8)))*7)/115;// format 12|4
    ram_data[BIOLOID_IMU_GYRO_X_L+i*2]=value&0xFF;
    ram_data[BIOLOID_IMU_GYRO_X_H+i*2]=(value>>8);
  }
}

